@@ -1,3 +1,10 @@
+// Copyright (c) 2020-2022 Weird Constructor <weirdconstructor@gmail.com>
+// This is a part of WLambda. See README.md and COPYING for details.
+
+/*!
+Implements the VM program and ops data structures for WLambda.
+*/
+
 use crate::vval::*;
 use crate::compiler::*;
 use crate::nvec::NVec;
@@ -8,9 +15,9 @@ use std::rc::Rc;
 
 #[derive(Clone, Default)]
 pub struct Prog {
-    pub debug:   std::vec::Vec<Option<SynPos>>,
-    pub data:    std::vec::Vec<VVal>,
-    pub ops:     std::vec::Vec<Op>,
+    pub debug:     std::vec::Vec<Option<SynPos>>,
+    pub data:      std::vec::Vec<VVal>,
+    pub(crate) ops:       std::vec::Vec<Op>,
     pub nxt_debug: Option<SynPos>,
 }
 
@@ -31,7 +38,7 @@ fn patch_respos_data(rp: &mut ResPos, idx: u16) {
 }
 
 impl Prog {
-    pub fn append(&mut self, mut prog: Prog) {
+    pub(crate) fn append(&mut self, mut prog: Prog) {
         let self_data_next_idx : u16 = self.data.len() as u16;
 
         for o in prog.ops.iter_mut() {
@@ -175,7 +182,7 @@ impl Prog {
                 Op::JmpIfN(p1, _) => {
                     patch_respos_data(p1, self_data_next_idx);
                 },
-                Op::OrJmp(p1, _, p2) => {
+                Op::OrJmp(p1, _, p2, _) => {
                     patch_respos_data(p1, self_data_next_idx);
                     patch_respos_data(p2, self_data_next_idx);
                 },
@@ -273,9 +280,9 @@ impl Prog {
         self.ops.append(&mut prog.ops);
     }
 
-    pub fn op_count(&self) -> usize { self.ops.len() }
+    pub(crate) fn op_count(&self) -> usize { self.ops.len() }
 
-    pub fn new() -> Self {
+    pub(crate) fn new() -> Self {
         Self {
             data:       vec![],
             ops:        vec![],
@@ -284,17 +291,17 @@ impl Prog {
         }
     }
 
-    pub fn global_ref_pos(&mut self, data: VVal) -> ResPos {
+    pub(crate) fn global_ref_pos(&mut self, data: VVal) -> ResPos {
         self.data.push(data);
         ResPos::GlobalRef((self.data.len() - 1) as u16)
     }
 
-    pub fn global_pos(&mut self, data: VVal) -> ResPos {
+    pub(crate) fn global_pos(&mut self, data: VVal) -> ResPos {
         self.data.push(data);
         ResPos::Global((self.data.len() - 1) as u16)
     }
 
-    pub fn data_pos(&mut self, data: VVal) -> ResPos {
+    pub(crate) fn data_pos(&mut self, data: VVal) -> ResPos {
         self.data.push(data);
         ResPos::Data((self.data.len() - 1) as u16)
     }
@@ -305,272 +312,272 @@ impl Prog {
 //        self
 //    }
 
-    pub fn push_op(&mut self, o: Op) -> &mut Self {
+    pub(crate) fn push_op(&mut self, o: Op) -> &mut Self {
         self.ops.push(o);
         self.debug.push(std::mem::replace(&mut self.nxt_debug, None));
         self
     }
 
-    pub fn set_dbg(&mut self, sp: SynPos) -> &mut Self {
+    pub(crate) fn set_dbg(&mut self, sp: SynPos) -> &mut Self {
         self.nxt_debug = Some(sp);
         self
     }
 
-    pub fn op_end(&mut self) -> &mut Self {
+    pub(crate) fn op_end(&mut self) -> &mut Self {
         self.push_op(Op::End);
         self
     }
 
-    pub fn op_mov(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
+    pub(crate) fn op_mov(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Mov(a, r));
     }
 
-    pub fn op_and_jmp(&mut self, sp: &SynPos, a: ResPos, jmp: i32, r: ResPos) {
+    pub(crate) fn op_and_jmp(&mut self, sp: &SynPos, a: ResPos, jmp: i32, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::AndJmp(a, jmp, r));
     }
 
-    pub fn op_or_jmp(&mut self, sp: &SynPos, a: ResPos, jmp: i32, r: ResPos) {
+    pub(crate) fn op_or_jmp_mode(&mut self, sp: &SynPos, a: ResPos, jmp: i32, r: ResPos, mode: OrMode) {
         self.set_dbg(sp.clone());
-        self.push_op(Op::OrJmp(a, jmp, r));
+        self.push_op(Op::OrJmp(a, jmp, r, mode));
     }
 
-    pub fn op_jmp_tbl(&mut self, sp: &SynPos, a: ResPos, tbl: Vec<i32>) {
+    pub(crate) fn op_jmp_tbl(&mut self, sp: &SynPos, a: ResPos, tbl: Vec<i32>) {
         self.set_dbg(sp.clone());
         self.push_op(Op::JmpTbl(a, Box::new(tbl)));
     }
 
-    pub fn op_destr(&mut self, sp: &SynPos, a: ResPos, destr_info: DestructureInfo) {
+    pub(crate) fn op_destr(&mut self, sp: &SynPos, a: ResPos, destr_info: DestructureInfo) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Destr(a, Box::new(destr_info)));
     }
 
-    pub fn op_to_ref(&mut self, sp: &SynPos, a: ResPos, r: ResPos, typ: ToRefType) {
+    pub(crate) fn op_to_ref(&mut self, sp: &SynPos, a: ResPos, r: ResPos, typ: ToRefType) {
         self.set_dbg(sp.clone());
         self.push_op(Op::ToRef(a, r, typ));
     }
 
-    pub fn op_new_list(&mut self, sp: &SynPos, r: ResPos) {
+    pub(crate) fn op_new_list(&mut self, sp: &SynPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewList(r));
     }
 
-    pub fn op_list_splice(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_list_splice(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::ListSplice(a, b, r));
     }
 
-    pub fn op_list_push(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_list_push(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::ListPush(a, b, r));
     }
 
-    pub fn op_new_map(&mut self, sp: &SynPos, r: ResPos) {
+    pub(crate) fn op_new_map(&mut self, sp: &SynPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewMap(r));
     }
 
-    pub fn op_map_set_key(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, r: ResPos) {
+    pub(crate) fn op_map_set_key(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::MapSetKey(a, b, c, r));
     }
 
-    pub fn op_map_splice(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_map_splice(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::MapSplice(a, b, r));
     }
 
-    pub fn op_get_key(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_get_key(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetKey(a, b, r));
     }
 
-    pub fn op_get_idx(&mut self, sp: &SynPos, a: ResPos, i: u32, r: ResPos) {
+    pub(crate) fn op_get_idx(&mut self, sp: &SynPos, a: ResPos, i: u32, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetIdx(a, i, r));
     }
 
-    pub fn op_get_idx2(&mut self, sp: &SynPos, a: ResPos, i: u32, i2: u32, r: ResPos) {
+    pub(crate) fn op_get_idx2(&mut self, sp: &SynPos, a: ResPos, i: u32, i2: u32, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetIdx2(a, Box::new((i, i2)), r));
     }
 
-    pub fn op_get_idx3(&mut self, sp: &SynPos, a: ResPos, i: u32, i2: u32, i3: u32, r: ResPos) {
+    pub(crate) fn op_get_idx3(&mut self, sp: &SynPos, a: ResPos, i: u32, i2: u32, i3: u32, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetIdx3(a, Box::new((i, i2, i3)), r));
     }
 
-    pub fn op_get_sym(&mut self, sp: &SynPos, a: ResPos, s: Symbol, r: ResPos) {
+    pub(crate) fn op_get_sym(&mut self, sp: &SynPos, a: ResPos, s: Symbol, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetSym(a, Box::new(s), r));
     }
 
-    pub fn op_get_sym2(&mut self, sp: &SynPos, a: ResPos, s: Symbol, s2: Symbol, r: ResPos) {
+    pub(crate) fn op_get_sym2(&mut self, sp: &SynPos, a: ResPos, s: Symbol, s2: Symbol, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetSym2(a, Box::new((s, s2)), r));
     }
 
-    pub fn op_get_sym3(&mut self, sp: &SynPos, a: ResPos, s: Symbol, s2: Symbol, s3: Symbol, r: ResPos) {
+    pub(crate) fn op_get_sym3(&mut self, sp: &SynPos, a: ResPos, s: Symbol, s2: Symbol, s3: Symbol, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::GetSym3(a, Box::new((s, s2, s3)), r));
     }
 
-    pub fn op_new_err(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
+    pub(crate) fn op_new_err(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewErr(a, r));
     }
 
-    pub fn op_argv(&mut self, sp: &SynPos, r: ResPos) {
+    pub(crate) fn op_argv(&mut self, sp: &SynPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Argv(r));
     }
 
-    pub fn op_unwind(&mut self, sp: &SynPos) {
+    pub(crate) fn op_unwind(&mut self, sp: &SynPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Unwind);
     }
 
-    pub fn op_binop(&mut self, sp: &SynPos, op: BinOp, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_binop(&mut self, sp: &SynPos, op: BinOp, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(op.to_op(a, b, r));
     }
 
-    pub fn op_new_opt(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
+    pub(crate) fn op_new_opt(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewOpt(a, r));
     }
 
-    pub fn op_new_iter(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
+    pub(crate) fn op_new_iter(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewIter(a, r));
     }
 
-    pub fn op_new_clos(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
+    pub(crate) fn op_new_clos(&mut self, sp: &SynPos, a: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewClos(a, r));
     }
 
-    pub fn op_call_method_key(&mut self, sp: &SynPos, a: ResPos, b: ResPos, argc: u16, r: ResPos) {
+    pub(crate) fn op_call_method_key(&mut self, sp: &SynPos, a: ResPos, b: ResPos, argc: u16, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::CallMethodKey(a, b, argc, r));
     }
 
-    pub fn op_call_method_sym(&mut self, sp: &SynPos, a: ResPos, sym: String, argc: u16, r: ResPos) {
+    pub(crate) fn op_call_method_sym(&mut self, sp: &SynPos, a: ResPos, sym: String, argc: u16, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::CallMethodSym(a, Box::new((sym, argc)), r));
     }
 
-    pub fn op_call_direct(&mut self, sp: &SynPos, a: ResPos, mut fun: DirectFun, r: ResPos) {
+    pub(crate) fn op_call_direct(&mut self, sp: &SynPos, a: ResPos, mut fun: DirectFun, r: ResPos) {
         self.set_dbg(sp.clone());
         fun.arg = a;
         fun.res = r;
         self.push_op(Op::CallDirect(Rc::new(fun)));
     }
 
-    pub fn op_call(&mut self, sp: &SynPos, argc: u16, r: ResPos) {
+    pub(crate) fn op_call(&mut self, sp: &SynPos, argc: u16, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Call(argc, r));
     }
 
-    pub fn op_new_ivec2(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_new_ivec2(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewNVec(Box::new(NVecPos::IVec2(a, b)), r));
     }
 
-    pub fn op_new_ivec3(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, r: ResPos) {
+    pub(crate) fn op_new_ivec3(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewNVec(Box::new(NVecPos::IVec3(a, b, c)), r));
     }
 
     #[allow(clippy::many_single_char_names)]
-    pub fn op_new_ivec4(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, d: ResPos, r: ResPos) {
+    pub(crate) fn op_new_ivec4(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, d: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewNVec(Box::new(NVecPos::IVec4(a, b, c, d)), r));
     }
 
-    pub fn op_new_fvec2(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_new_fvec2(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewNVec(Box::new(NVecPos::FVec2(a, b)), r));
     }
 
-    pub fn op_new_fvec3(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, r: ResPos) {
+    pub(crate) fn op_new_fvec3(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewNVec(Box::new(NVecPos::FVec3(a, b, c)), r));
     }
 
     #[allow(clippy::many_single_char_names)]
-    pub fn op_new_fvec4(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, d: ResPos, r: ResPos) {
+    pub(crate) fn op_new_fvec4(&mut self, sp: &SynPos, a: ResPos, b: ResPos, c: ResPos, d: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewNVec(Box::new(NVecPos::FVec4(a, b, c, d)), r));
     }
 
-    pub fn op_clear_locals(&mut self, sp: &SynPos, from: u16, to: u16) {
+    pub(crate) fn op_clear_locals(&mut self, sp: &SynPos, from: u16, to: u16) {
         self.set_dbg(sp.clone());
         self.push_op(Op::ClearLocals(from, to));
     }
 
-    pub fn op_ctrl_flow_break(&mut self, sp: &SynPos, a: ResPos) {
+    pub(crate) fn op_ctrl_flow_break(&mut self, sp: &SynPos, a: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::CtrlFlow(CtrlFlow::Break(a)));
     }
 
-    pub fn op_ctrl_flow_next(&mut self, sp: &SynPos) {
+    pub(crate) fn op_ctrl_flow_next(&mut self, sp: &SynPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::CtrlFlow(CtrlFlow::Next));
     }
 
-    pub fn op_jmp(&mut self, sp: &SynPos, offs: i32) {
+    pub(crate) fn op_jmp(&mut self, sp: &SynPos, offs: i32) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Jmp(offs));
     }
 
-    pub fn op_jmp_ifn(&mut self, sp: &SynPos, a: ResPos, offs: i32) {
+    pub(crate) fn op_jmp_ifn(&mut self, sp: &SynPos, a: ResPos, offs: i32) {
         self.set_dbg(sp.clone());
         self.push_op(Op::JmpIfN(a, offs));
     }
 
-    pub fn op_push_loop_info(&mut self, sp: &SynPos, break_offs: u16) {
+    pub(crate) fn op_push_loop_info(&mut self, sp: &SynPos, break_offs: u16) {
         self.set_dbg(sp.clone());
         self.push_op(Op::PushLoopInfo(break_offs));
     }
 
-    pub fn op_iter_init(&mut self, sp: &SynPos, a: ResPos, end_offs: i32) {
+    pub(crate) fn op_iter_init(&mut self, sp: &SynPos, a: ResPos, end_offs: i32) {
         self.set_dbg(sp.clone());
         self.push_op(Op::IterInit(a, end_offs));
     }
 
-    pub fn op_iter_next(&mut self, sp: &SynPos, r: ResPos) {
+    pub(crate) fn op_iter_next(&mut self, sp: &SynPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::IterNext(r));
     }
 
-    pub fn op_apply(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_apply(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Apply(a, b, r));
     }
 
-    pub fn op_dump_vm(&mut self, sp: &SynPos) {
+    pub(crate) fn op_dump_vm(&mut self, sp: &SynPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Builtin(Builtin::DumpVM(Box::new(sp.clone()))));
     }
 
-    pub fn op_dump_stack(&mut self, sp: &SynPos) {
+    pub(crate) fn op_dump_stack(&mut self, sp: &SynPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Builtin(Builtin::DumpStack(Box::new(sp.clone()))));
     }
 
-    pub fn op_export(&mut self, sp: &SynPos, a: ResPos, name: String) {
+    pub(crate) fn op_export(&mut self, sp: &SynPos, a: ResPos, name: String) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Builtin(Builtin::Export(Box::new(name), a)));
     }
 
-    pub fn op_new_pair(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
+    pub(crate) fn op_new_pair(&mut self, sp: &SynPos, a: ResPos, b: ResPos, r: ResPos) {
         self.set_dbg(sp.clone());
         self.push_op(Op::NewPair(a, b, r));
     }
 
-    pub fn op_accumulator(&mut self, sp: &SynPos, at: AccumType) {
+    pub(crate) fn op_accumulator(&mut self, sp: &SynPos, at: AccumType) {
         self.set_dbg(sp.clone());
         self.push_op(Op::Accumulator(at));
     }
@@ -602,7 +609,7 @@ impl Debug for Prog {
 }
 
 #[derive(Debug, Clone)]
-pub struct DestructureInfo {
+pub(crate) struct DestructureInfo {
     pub vars:   VVal,
     pub poses:  std::vec::Vec<VarPos>,
     pub is_ref: bool,
@@ -659,7 +666,7 @@ impl DestructureInfo {
             VVal::Map(m) => {
                 for (i, pos) in self.poses.iter().enumerate() {
                     let sym = self.vars.at(i).unwrap().to_sym();
-                    let val = m.borrow().get(&sym).cloned().unwrap_or_else(|| VVal::None);
+                    let val = m.borrow().get(&sym).cloned().unwrap_or(VVal::None);
 
                     set_at_varpos!(self, env, pos, &val);
                 }
@@ -668,11 +675,11 @@ impl DestructureInfo {
                 let (lv, rv) = &*p;
 
                 if let Some(pos) = self.poses.get(0) {
-                    set_at_varpos!(self, env, pos, &lv);
+                    set_at_varpos!(self, env, pos, lv);
                 }
 
                 if let Some(pos) = self.poses.get(1) {
-                    set_at_varpos!(self, env, pos, &rv);
+                    set_at_varpos!(self, env, pos, rv);
                 }
             },
             VVal::IVec(vb) => {
@@ -771,7 +778,7 @@ impl DestructureInfo {
 }
 
 #[derive(Debug, Clone, Copy)]
-pub enum BinOp {
+pub(crate) enum BinOp {
     Add,
     Sub,
     Mul,
@@ -785,24 +792,24 @@ pub enum BinOp {
 }
 
 impl BinOp {
-    pub fn to_op(self, a: ResPos, b: ResPos, out: ResPos) -> Op {
+    pub(crate) fn to_op(self, a: ResPos, b: ResPos, out: ResPos) -> Op {
         match self {
-            BinOp::Add => Op::Add(a, b, out),
-            BinOp::Sub => Op::Sub(a, b, out),
-            BinOp::Mul => Op::Mul(a, b, out),
-            BinOp::Div => Op::Div(a, b, out),
-            BinOp::Mod => Op::Mod(a, b, out),
-            BinOp::Le  => Op::Le(a, b, out),
-            BinOp::Lt  => Op::Lt(a, b, out),
-            BinOp::Ge  => Op::Ge(a, b, out),
-            BinOp::Gt  => Op::Gt(a, b, out),
-            BinOp::Eq  => Op::Eq(a, b, out),
+            BinOp::Add       => Op::Add(a, b, out),
+            BinOp::Sub       => Op::Sub(a, b, out),
+            BinOp::Mul       => Op::Mul(a, b, out),
+            BinOp::Div       => Op::Div(a, b, out),
+            BinOp::Mod       => Op::Mod(a, b, out),
+            BinOp::Le        => Op::Le(a, b, out),
+            BinOp::Lt        => Op::Lt(a, b, out),
+            BinOp::Ge        => Op::Ge(a, b, out),
+            BinOp::Gt        => Op::Gt(a, b, out),
+            BinOp::Eq        => Op::Eq(a, b, out),
         }
     }
 }
 
 #[derive(Debug,Clone,Copy)]
-pub enum AccumType {
+pub(crate) enum AccumType {
     String,
     Bytes,
     Float,
@@ -812,7 +819,7 @@ pub enum AccumType {
 }
 
 #[derive(Debug,Clone)]
-pub enum ToRefType {
+pub(crate) enum ToRefType {
     CaptureRef,
     ToRef,
     Deref,
@@ -821,20 +828,21 @@ pub enum ToRefType {
 }
 
 #[derive(Debug,Clone)]
-pub enum Builtin {
+#[allow(clippy::box_collection)]
+pub(crate) enum Builtin {
     Export(Box<String>, ResPos),
     DumpStack(Box<SynPos>),
     DumpVM(Box<SynPos>),
 }
 
 #[derive(Debug,Clone)]
-pub enum CtrlFlow {
+pub(crate) enum CtrlFlow {
     Next,
     Break(ResPos),
 }
 
 #[derive(Debug,Clone)]
-pub enum NVecPos {
+pub(crate) enum NVecPos {
     IVec2(ResPos, ResPos),
     IVec3(ResPos, ResPos, ResPos),
     IVec4(ResPos, ResPos, ResPos, ResPos),
@@ -844,7 +852,7 @@ pub enum NVecPos {
 }
 
 #[derive(Clone)]
-pub struct DirectFun {
+pub(crate) struct DirectFun {
     pub arg: ResPos,
     pub res: ResPos,
     pub fun: Rc<dyn Fn(VVal, &mut Env) -> VVal>,
@@ -866,9 +874,20 @@ impl Debug for DirectFun {
     }
 }
 
-#[allow(clippy::box_vec)]
+#[derive(Debug,Clone,Copy)]
+#[repr(u8)]
+pub(crate) enum OrMode {
+    Bool,
+    SomeOp,
+    ExtSomeOp,
+    NoneOp,
+    ErrOp,
+    OptOp,
+}
+
+#[allow(clippy::box_collection)]
 #[derive(Debug,Clone)]
-pub enum Op {
+pub(crate) enum Op {
     Mov(ResPos, ResPos),
     NewOpt(ResPos, ResPos),
     NewIter(ResPos, ResPos),
@@ -912,7 +931,7 @@ pub enum Op {
     Apply(ResPos, ResPos, ResPos),
     Jmp(i32),
     JmpIfN(ResPos, i32),
-    OrJmp(ResPos, i32, ResPos),
+    OrJmp(ResPos, i32, ResPos, OrMode),
     AndJmp(ResPos, i32, ResPos),
     JmpTbl(ResPos, Box<Vec<i32>>),
     CtrlFlow(CtrlFlow),
