@@ -1,4 +1,4 @@
-// Copyright (c) 2020 Weird Constructor <weirdconstructor@gmail.com>
+// Copyright (c) 2020-2022 Weird Constructor <weirdconstructor@gmail.com>
 // This is a part of WLambda. See README.md and COPYING for details.
 
 use crate::vval::VVal;
@@ -125,9 +125,10 @@ impl fmt::Display for ParseValueError {
         }
     }
 }
-impl Into<ParseErrorKind> for ParseValueError {
-    fn into(self) -> ParseErrorKind {
-        ParseErrorKind::BadValue(self)
+
+impl From<ParseValueError> for ParseErrorKind {
+    fn from(p: ParseValueError) -> ParseErrorKind {
+        ParseErrorKind::BadValue(p)
     }
 }
 
@@ -160,9 +161,10 @@ impl fmt::Display for ParseNumberError {
         }
     }
 }
-impl Into<ParseErrorKind> for ParseNumberError {
-    fn into(self) -> ParseErrorKind {
-        ParseErrorKind::BadNumber(self)
+
+impl From<ParseNumberError> for ParseErrorKind {
+    fn from(p: ParseNumberError) -> ParseErrorKind {
+        ParseErrorKind::BadNumber(p)
     }
 }
 
@@ -178,15 +180,18 @@ pub struct ParseError {
 
 impl fmt::Display for ParseError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(
-            f,
-            "error[{},{}:{}] {} at code '{}'",
+        writeln!(f, "{}:{}:{} {}",
+            self.file.s(),
             self.line,
             self.col,
-            self.file.s(),
-            self.kind,
-            self.snip
-        )
+            self.kind)?;
+
+        writeln!(f, "at code:")?;
+        for (i, line) in self.snip.split('\n').enumerate() {
+            writeln!(f, "{:<4}| {}", self.line as usize + i, line)?
+        }
+
+        Ok(())
     }
 }
 
@@ -232,10 +237,14 @@ impl State {
         }
     }
 
+    /// Creates a `SynPos` annotated with the current parse head position.
+    pub fn syn_pos(&self, s: Syntax) -> SynPos {
+        SynPos::new(s, self.line_no, self.col_no, self.file.clone())
+    }
 
     /// Creates a `VVal::Syn` annotated with the current parse head position.
     pub fn syn_raw(&self, s: Syntax) -> VVal {
-        VVal::Syn(SynPos::new(s, self.line_no, self.col_no, self.file.clone()))
+        VVal::Syn(self.syn_pos(s))
     }
 
     /// Creates an syntactic AST node.
@@ -342,8 +351,16 @@ impl State {
                 },
             '*' | '/' | '%' | '^'
                 => {
+                    if let Some(s) = self.peek3() {
+                        if    s == "/$e"
+                           || s == "/$n"
+                           || s == "/$o" { return Some(s); }
+                    }
                     if let Some(s) = self.peek2() {
-                        if s == "%>" {
+                        if    s == "%>"
+                           || s == "//"
+                           || s == "/?"
+                        {
                             Some(s)
                         } else {
                             Some(self.spart_ptr(1))
