@@ -1,4 +1,4 @@
-// Copyright (c) 2021 Weird Constructor <weirdconstructor@gmail.com>
+// Copyright (c) 2021-2022 Weird Constructor <weirdconstructor@gmail.com>
 // This is a part of WLambda. See README.md and COPYING for details.
 use crate::compiler::*;
 use crate::vval::*;
@@ -7,7 +7,7 @@ use std::cell::RefCell;
 
 use std::net::{SocketAddr, ToSocketAddrs, UdpSocket};
 
-fn vv2socketaddr(vv: &VVal) -> Result<Box<dyn std::iter::Iterator<Item = SocketAddr>>, String> {
+pub fn vv2socketaddr(vv: &VVal) -> Result<Box<dyn std::iter::Iterator<Item = SocketAddr>>, String> {
     let addr =
         if vv.is_pair() {
             let port = vv.v_i(1) as u16;
@@ -53,7 +53,7 @@ impl VValUserData for VTcpStream {
             "$<TcpStream:local={}/remote={}>",
             self.stream.borrow().local_addr()
                 .map(|a| a.to_string())
-                .map_or_else(|_| String::from("?"), |a| a.to_string()),
+                .map_or_else(|_| String::from("?"), |a| a),
             self.stream.borrow().peer_addr()
                 .map_or_else(|_| String::from("?"), |a| a.to_string()))
     }
@@ -101,7 +101,7 @@ impl VValUserData for VUdpSocket {
             "$<UdpSocket:local={}/remote={}>",
             self.socket.borrow().local_addr()
                 .map(|a| a.to_string())
-                .map_or_else(|_| String::from("?"), |a| a.to_string()),
+                .map_or_else(|_| String::from("?"), |a| a),
             self.socket.borrow().peer_addr()
                 .map_or_else(|_| String::from("?"), |a| a.to_string()))
     }
@@ -155,13 +155,38 @@ macro_rules! try_addresses {
     }
 }
 
+#[macro_export]
+macro_rules! first_addr {
+    ($arg: expr, $env: expr) => {
+        {
+            let vaddr = $arg;
+
+            let mut addr =
+                match crate::stdlib::net::vv2socketaddr(&vaddr) {
+                    Ok(addr) => addr,
+                    Err(e) => { return Err($env.new_err(e)) },
+                };
+
+            let first = addr.next();
+            if first.is_none() {
+                return Err($env.new_err(format!(
+                    "Couldn't get socket address from '{}'",
+                    vaddr.s_raw())));
+            }
+
+            Ok(first.unwrap())
+        }
+    }
+}
+
+#[macro_export]
 macro_rules! with_first_addr {
     ($arg: expr, $env: expr, $addr: ident, $block: tt) => {
         {
             let vaddr = $arg;
 
             let mut addr =
-                match vv2socketaddr(&vaddr) {
+                match crate::stdlib::net::vv2socketaddr(&vaddr) {
                     Ok(addr) => addr,
                     Err(e) => { return Ok($env.new_err(e)) },
                 };
@@ -190,15 +215,9 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                     let listener = {
                         use socket2::Socket;
                         let socket = Socket::from(listener);
-                        match socket.set_reuse_address(true) {
-                            Err(_) => {},
-                            Ok(_)  => {},
-                        }
+                        let _ = socket.set_reuse_address(true);
                         #[cfg(unix)]
-                        match socket.set_reuse_port(true) {
-                            Err(_) => {},
-                            Ok(_)  => {},
-                        }
+                        let _ = socket.set_reuse_port(true);
                         socket.into_tcp_listener()
                     };
 
@@ -326,7 +345,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                     }
                 }
             })
-        }).unwrap_or(
+        }).unwrap_or_else(||
             Ok(env.new_err(format!(
                 "std:net:udp:send: First argument not a socket! {}",
                 socket.s()))))
@@ -352,7 +371,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                 },
                 Err(e) => env.new_err(format!("std:net:udp:recv: {}", e)),
             }
-        }).unwrap_or(
+        }).unwrap_or_else(||
             env.new_err(format!(
                 "std:net:udp:recv: First argument not a socket! {}",
                 socket.s()))))
@@ -367,7 +386,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                 Ok(_)  => VVal::Bol(true),
                 Err(e) => env.new_err(format!("std:io:flush: {}", e)),
             }
-        }).unwrap_or(
+        }).unwrap_or_else(||
             env.new_err(format!(
                 "std:io:flush: First argument not an IO handle! {}",
                 fd.s()))))
@@ -384,7 +403,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
             data.with_bv_ref(|bytes| {
                 if offs >= bytes.len() {
                     return env.new_err(
-                        format!("std:io:write_some: bad buffer offset"));
+                        "std:io:write_some: bad buffer offset".to_string());
                 }
 
                 let r = vts.stream.borrow_mut().write_all(&bytes[offs..]);
@@ -402,7 +421,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                     }
                 }
             })
-        }).unwrap_or(
+        }).unwrap_or_else(||
             env.new_err(format!(
                 "std:io:write: First argument not an IO handle! {}",
                 fd.s()))))
@@ -419,7 +438,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
             data.with_bv_ref(|bytes| {
                 if offs >= bytes.len() {
                     return env.new_err(
-                        format!("std:io:write_some: bad buffer offset"));
+                        "std:io:write_some: bad buffer offset".to_string());
                 }
 
                 let r = vts.stream.borrow_mut().write(&bytes[offs..]);
@@ -445,7 +464,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                     }
                 }
             })
-        }).unwrap_or(
+        }).unwrap_or_else(||
             env.new_err(format!(
                 "std:io:write_some: First argument not an IO handle! {}",
                 fd.s()))))
@@ -479,7 +498,7 @@ pub fn add_to_symtable(st: &mut SymbolTable) {
                     }
                 }
             }
-        }).unwrap_or(
+        }).unwrap_or_else(||
             env.new_err(format!(
                 "std:io:read_some: First argument not an IO handle! {}",
                 fd.s()))))
