@@ -1,4 +1,4 @@
-// Copyright (c) 2020 Weird Constructor <weirdconstructor@gmail.com>
+// Copyright (c) 2020-2022 Weird Constructor <weirdconstructor@gmail.com>
 // This is a part of WLambda. See README.md and COPYING for details.
 
 use crate::VVal;
@@ -232,7 +232,7 @@ pub(crate) fn read_sax(env: &mut Env, input: VVal, event_func: VVal, trim: bool)
                     Ok(v)                      => { ret = v; },
                     Err(StackAction::Break(v)) => { ret = *v; break; },
                     Err(StackAction::Next)     => { },
-                    Err(e)                     => { return Err(e); },
+                    Err(e)                     => { env.popn(1); return Err(e); },
                 };
                 env.popn(1);
             }
@@ -287,7 +287,7 @@ pub(crate) fn create_sax_writer(indent_depth: Option<usize>) -> VVal {
 
             let elem = env.arg(0);
             elem.v_with_s_ref(0, |s| {
-                match &s[..] {
+                match s {
                     "start" => {
                         elem.v_with_s_ref(1, |name| {
                             let mut bytes =
@@ -428,7 +428,7 @@ impl VValBuilder {
     //
     pub fn event(&mut self, elem: &VVal) {
         elem.v_with_s_ref(0, |s| {
-            match &s[..] {
+            match s {
                 "start" => {
                     self.stack.push(std::mem::replace(&mut self.cur,
                         VVal::vec4(
