@@ -1,4 +1,4 @@
-// Copyright (c) 2020 Weird Constructor <weirdconstructor@gmail.com>
+// Copyright (c) 2020-2022 Weird Constructor <weirdconstructor@gmail.com>
 // This is a part of WLambda. See README.md and COPYING for details.
 
 /*!
@@ -472,7 +472,8 @@ impl VValUserData for AtomicAValSlot {
                     return
                         Err(StackAction::panic_str(
                             "recv_timeout method expects 1 argument".to_string(),
-                            None))
+                            None,
+                            env.argv()))
                 }
 
                 Ok(self.recv_timeout(Some(argv[0].to_duration()?)))
@@ -482,7 +483,8 @@ impl VValUserData for AtomicAValSlot {
                     return
                         Err(StackAction::panic_str(
                             "send method expects 1 argument".to_string(),
-                            None))
+                            None,
+                            env.argv()))
                 }
 
                 Ok(self.send(&argv[0]))
@@ -494,7 +496,8 @@ impl VValUserData for AtomicAValSlot {
                     return
                         Err(StackAction::panic_str(
                             "wait_empty_timeout method expects 1 argument".to_string(),
-                            None))
+                            None,
+                            env.argv()))
                 }
 
                 Ok(self.wait_empty(Some(argv[0].to_duration()?)))
@@ -502,7 +505,8 @@ impl VValUserData for AtomicAValSlot {
             _ => {
                 Err(StackAction::panic_str(
                     format!("unknown method called: {}", key),
-                    None))
+                    None,
+                    env.argv()))
             },
         }
     }
@@ -520,6 +524,12 @@ pub struct AValChannel {
 #[derive(Clone, Debug)]
 pub struct ForkSenderError(String);
 
+impl std::fmt::Display for ForkSenderError {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        write!(f, "Failed to fork sender, can't get lock: {}", self.0)
+    }
+}
+
 impl AValChannel {
     pub(crate) fn new_direct() -> AValChannel {
         let (send, recv) = std::sync::mpsc::channel();
@@ -630,7 +640,8 @@ impl VValUserData for AValChannel {
                     return
                         Err(StackAction::panic_str(
                             "recv_timeout method expects 1 argument".to_string(),
-                            None))
+                            None,
+                            env.argv()))
                 }
 
                 Ok(self.recv_timeout(Some(argv[0].to_duration()?)))
@@ -640,7 +651,8 @@ impl VValUserData for AValChannel {
                     return
                         Err(StackAction::panic_str(
                             "send method expects 1 argument".to_string(),
-                            None))
+                            None,
+                            env.argv()))
                 }
 
                 Ok(self.send(&argv[0]))
@@ -648,7 +660,8 @@ impl VValUserData for AValChannel {
             _ => {
                 Err(StackAction::panic_str(
                     format!("unknown method called: {}", key),
-                    None))
+                    None,
+                    env.argv()))
             },
         }
     }
@@ -726,7 +739,8 @@ impl VValUserData for AtomicAVal {
             _ => {
                 Err(StackAction::panic_str(
                     format!("Unknown method called: {}", key),
-                    None))
+                    None,
+                    env.argv()))
             },
         }
     }
@@ -941,7 +955,8 @@ impl VValUserData for DefaultThreadHandle {
             _ => {
                 Err(StackAction::panic_str(
                     format!("Unknown method called: {}", key),
-                    None))
+                    None,
+                    env.argv()))
             },
         }
     }
